import click
import os
from .config import yaml_load
from .compile import compile as cm


@click.command()
@click.option('--config-path', default="sm-config.yml", help='Configuration file in YAML.')
def compile_schema(config_path):
    cwd = os.getcwd()
    try:
        workdir = os.path.dirname(config_path)
        file_path = os.path.basename(config_path)
        if workdir != '':
            os.chdir(workdir)
        config = yaml_load(open(file_path).read())
        cm(config)
    finally:
        os.chdir(cwd)



import pytest  # noqa: F401
import os
import shutil
import tempfile
import schemamacros.compile as c
import schemamacros.config as cfg
from jinja2 import loaders, Environment


def test_build_loader(config_dict):
    config = cfg.build_config_inner(config_dict)
    loader = c.build_loader(config.template_directories,
                            config.template_packages)

    assert(isinstance(loader, loaders.ChoiceLoader))

    env = Environment(loader=loader)
    tpl = env.get_template("schema.sql")

    assert(tpl is not None)


def test_render(config_dict):
    config = cfg.build_config_inner(config_dict)
    schema_texts = c.render(config)
    print(schema_texts)
    schema_text = schema_texts["schema_out.sql"].strip()

    assert(schema_text != '')
    assert("FOO" in schema_text)


def test_compile(config_dict):
    config = cfg.build_config_inner(config_dict)
    schema_text = c.render(config)["schema_out.sql"].strip()

    with tempfile.NamedTemporaryFile('r') as f:
        config_tmp_dict = dict(config_dict)
        config_tmp_dict["output"] = f.name

        config_tmp = cfg.build_config_inner(config_tmp_dict)
        c.compile(config_tmp)
        schema_text_c = f.read().strip()

    assert(schema_text != '')
    assert(schema_text_c != '' and type(schema_text_c) == str)

    assert(schema_text_c == schema_text)


def test_module_templates(config_dict_modules):
    config = cfg.build_config_inner(config_dict_modules)
    schema_text = c.render(config)["schema_out.sql"].strip()

    with tempfile.NamedTemporaryFile('r') as f:
        config_tmp_dict = dict(config_dict_modules)
        config_tmp_dict["output"] = f.name

        config_tmp = cfg.build_config_inner(config_tmp_dict)
        c.compile(config_tmp)
        schema_text_c = f.read().strip()

    assert(schema_text_c != '' and type(schema_text_c) == str)
    assert(schema_text != '')
    assert(schema_text_c == schema_text)


def test_compile_cwd(config_dict):
    config = cfg.build_config_inner(config_dict)
    schema_text = c.render(config)["schema_out.sql"].strip()
    saved_path = os.getcwd()
    try:
        with tempfile.TemporaryDirectory() as t_dir:
            shutil.copytree('tests/',
                            os.path.join(t_dir, 'tests'))
            os.chdir(t_dir)
            c.compile(config)
            f = open("schema_out.sql")
            schema_text_c = f.read().strip()
    finally:
        os.chdir(saved_path)

    assert(schema_text != '')
    assert(schema_text_c != '' and type(schema_text_c) == str)
    assert(schema_text_c == schema_text)


def test_render_config11(config11_yaml):
    config = cfg.yaml_load(config11_yaml)
    schema_texts = c.render(config)

    schema_shadowed_text = schema_texts["schema_shadowed_out.sql"]
    schema_text = schema_texts["schema_out.sql"]

    assert(schema_shadowed_text != '')
    assert(schema_text != '')
    assert(schema_shadowed_text != schema_text)
    assert("True" in schema_shadowed_text)
    assert("False" in schema_text)



import pytest
import yaml


@pytest.fixture
def config_dict():
    return {
        "template_directories": [
            "tests/schema/templates",
        ],
        "template_packages": [],
        "variables": {
            "test_var": "FOO"
        },
        "transaction": True,
        "output": "schema_out.sql",
        "version": "1.0",
        "schema_template": "schema.sql"
    }


@pytest.fixture
def config_dict_modules():
    return {
        "template_directories": [],
        "template_packages": [
            {
                "package_name": "tests.schema",
                "template_path": "templates"
            }
        ],
        "variables": {
            "test_var": "FOO"
        },
        "transaction": True,
        "output": "schema_out.sql",
        "version": "1.0",
        "schema_template": "schema.sql"
    }


@pytest.fixture
def config_dict_yaml(config_dict):
    return yaml.dump(config_dict)


@pytest.fixture
def config_yaml():
    return """
version: "1"
template_directories:
    - "tests/schema/templates"
template_packages: []
transaction: True
output: schema_out.sql
variables:
    test_var: "FOO"
schema_template: "schema.sql"
"""


@pytest.fixture
def config11_yaml():
    return """
version: "1.1"
template_directories:
    - "tests/schema/templates"

template_packages: []

targets:
    schema_shadowed_out.sql:
        transaction: True
        schema_template: "schema.sql"
        variables:
            shadowed: "True"

    schema_out.sql:
        schema_template: "schema.sql"

variables:
    test_var: "FOO"
    shadowed: "False"

"""



import pytest  # noqa: F401
import schemamacros.config as c


def test_templatedir_exists():
    td = c.TemplateDir("/")
    assert(str(td) == '/')


def test_templatedir_nexists():
    ok = False
    try:
        td = c.TemplateDir("%")  # noqa: F841
    except ValueError:
        ok = True
    assert(ok)


def test_config_dict(config_dict):
    cfg = c.build_config_inner(config_dict)
    assert(cfg is not None)


def test_config_dict_yaml(config_dict, config_dict_yaml):
    c_yaml = c.yaml_load(config_dict_yaml)
    c_dict = c.build_config_inner(config_dict)
    assert(c_yaml == c_dict)
    assert(c_yaml is not None)


def test_config_yaml(config_yaml):
    c_yaml = c.yaml_load(config_yaml)
    assert(c_yaml is not None)

    assert(len(c_yaml.targets.items()) == 1)

    assert(len(c_yaml.template_directories) == 1)
    assert(c_yaml.template_directories == ["tests/schema/templates"])
    assert(len(c_yaml.template_packages) == 0)



import typing
from .config import ConfigInternal, TargetConfig
from jinja2 import loaders, Environment


header_tpl = """
-- Generated with schemamacros
"""

schema_block_tpl = """
{% block schema %}
{% endblock %}
"""


def build_layout(transaction, headers=[header_tpl], before_schema="", after_schema=""):
    headers_text = "\n".join(headers)
    if transaction:
        before_schema += "BEGIN;\n"
        after_schema += "COMMIT;\n"

    return headers_text + before_schema + schema_block_tpl + after_schema


def build_loader(paths, packages):
    fs_loaders = [loaders.FileSystemLoader(path) for path in paths]
    package_loaders = [loaders.PackageLoader(package.package_name, package.template_path)
                       for package in packages]

    return loaders.ChoiceLoader(fs_loaders + package_loaders)


def render_abstract(template: str,
                    loader: loaders.BaseLoader,
                    variables: typing.Mapping[str, str]):

    env = Environment(loader=loader)
    schema_tpl = env.get_template(template)
    return schema_tpl.render(variables)


def render_target(project_loader: loaders.BaseLoader,
                  project_variables: typing.Mapping[str, str],
                  target: TargetConfig):

    variables = dict(project_variables)
    variables.update(target.variables)

    layout = build_layout(target.transaction)
    layout_loader = loaders.DictLoader({"layout": layout})
    target_loader = loaders.ChoiceLoader([project_loader, layout_loader])

    render_text = render_abstract(target.schema_template,
                                  target_loader,
                                  variables)
    return render_text


def render(config: ConfigInternal):
    loader = build_loader(config.template_directories,
                          config.template_packages)
    output = {outpath: render_target(loader,
                                     config.variables,
                                     target)
              for outpath, target in config.targets.items()}

    return output


def compile(config: ConfigInternal):
    for outpath, schema_text in render(config).items():
        with open(outpath, 'w') as f:
            f.write(schema_text)
            f.flush()



import pytest  # noqa: F401
import tempfile
import shutil
import os
import schemamacros.cli as cli

from click.testing import CliRunner


def test_cli_compile():
    cwd = os.getcwd()
    try:
        with tempfile.TemporaryDirectory() as td:
            dst = os.path.join(td, 'schema')
            shutil.copytree('tests/schema', dst)
            os.chdir(dst)
            runner = CliRunner()
            result = runner.invoke(cli.compile_schema, [])
            assert(result.exit_code == 0)
            output_path = os.path.join(dst, 'schema-out.sql')
            output = open(output_path).read().strip()
        assert(len(output) > 0)
    finally:
        os.chdir(cwd)


def test_cli_compile_arg():
    cwd = os.getcwd()
    try:
        with tempfile.TemporaryDirectory() as td:
            dst = os.path.join(td, 'schema')
            shutil.copytree('tests/schema', dst)
            cfg_path = os.path.join(dst, 'sm-config.yml')
            runner = CliRunner()
            result = runner.invoke(cli.compile_schema, ['--config-path', cfg_path])
            assert(result.exit_code == 0)
            output_path = os.path.join(dst, 'schema-out.sql')
            output = open(output_path).read().strip()
        assert(len(output) > 0)
    finally:
        os.chdir(cwd)



"""
Schema Macros
----------------

Jinja2 enhanced SQL

"""

from setuptools import setup, find_packages


setup(
    name='schemamacros',
    version='0.2.0',
    url='https://github.com/caj-larsson/schemamacros',
    license='MIT',
    author='Caj Larsson',
    author_email='contact@caj.me',
    description='Template SQL',
    long_description=__doc__,
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Click',
        'attrs',
        'jinja2',
        'pyaml'
    ],
    entry_points='''
        [console_scripts]
        sm-compile=schemamacros.cli:compile_schema
    ''',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



import typing
import attr
from attr import validators as v
import yaml
import os


@attr.s(repr=False)
class TemplateDir(str):

    def __repr__(self):
        return self.path

    def __str__(self):
        return self.__repr__()

    path: str = attr.ib()

    @path.validator
    def check(self, attribute, value):
        if not os.path.isdir(value):
            raise ValueError("The path %s is not a directory", value)


@attr.s
class TemplatePackage(object):
    package_name: str = attr.ib(validator=v.instance_of(str))
    template_path: str = attr.ib(validator=v.instance_of(str))

    @classmethod
    def from_dict(cls, config_obj):
        return cls(**config_obj)


@attr.s
class TargetConfig(object):
    schema_template: str = attr.ib(validator=v.instance_of(str))
    transaction: bool = attr.ib(
        validator=v.instance_of(bool),
        default=False
    )
    variables: typing.Mapping[str, str] = attr.ib(default={})


@attr.s
class ConfigInternal(object):
    template_directories: typing.List[TemplateDir] = attr.ib(
        validator=v.instance_of(list),
        convert=lambda x: list(map(TemplateDir, x)),
        default=[]
    )

    template_packages: typing.List[TemplatePackage] = attr.ib(
        validator=v.instance_of(list),
        convert=lambda x: list(map(TemplatePackage.from_dict, x)),
        default=[]
    )

    variables: typing.Mapping[str, str] = attr.ib(default=dict())

    targets: typing.Mapping[str, TargetConfig] = attr.ib(default=dict())


@attr.s
class ConfigfileVersion1(object):
    version: str = attr.ib(validator=v.in_(["1", "1.0"]))

    schema_template: str = attr.ib(validator=v.instance_of(str))

    output: str = attr.ib(validator=attr.validators.instance_of(str))

    template_directories: typing.List[TemplateDir] = attr.ib(
        validator=v.instance_of(list)
    )

    transaction: bool = attr.ib(
        validator=v.instance_of(bool),
        default=False
    )

    variables: typing.Mapping[str, str] = attr.ib(default=dict())

    template_packages: typing.List[TemplatePackage] = attr.ib(
        validator=v.instance_of(list),
        default=[]
    )

    def extract(self) -> ConfigInternal:
        target_config = TargetConfig(
            transaction=self.transaction,
            schema_template=self.schema_template,
            variables={}
        )
        config = ConfigInternal(
            template_directories=self.template_directories,
            template_packages=self.template_packages,
            variables=self.variables,
            targets={self.output: target_config}
        )
        return config


@attr.s
class ConfigfileVersion11(object):
    version: str = attr.ib(validator=v.in_(["1.1"]))
    template_directories: typing.List[TemplateDir] = attr.ib(
        validator=v.instance_of(list),
        default=[]
    )

    template_packages: typing.List[TemplatePackage] = attr.ib(
        validator=v.instance_of(list),
        default=[]
    )

    variables: typing.Mapping[str, str] = attr.ib(default=dict())

    targets: typing.Mapping[str, TargetConfig] = attr.ib(default=dict())

    def extract(self) -> ConfigInternal:
        print(self.targets)

        config = ConfigInternal(
            template_directories=self.template_directories,
            template_packages=self.template_packages,
            variables=self.variables,
            targets={t: TargetConfig(**v)
                     for t, v in self.targets.items()}
        )
        return config


CONFIG_FILE_VERSIONS = {
    "1": ConfigfileVersion1,
    "1.0": ConfigfileVersion1,
    "1.1": ConfigfileVersion11
}


def build_config_inner(config_dict) -> ConfigInternal:
    if "version" not in config_dict:
        raise ValueError("Config does not have a version")

    version = str(config_dict["version"])

    if version not in CONFIG_FILE_VERSIONS:
        raise ValueError("Version %s is not supported", version)

    versioned_config = CONFIG_FILE_VERSIONS[version](**config_dict)

    return versioned_config.extract()


def yaml_load(yaml_text):
    return build_config_inner(yaml.load(yaml_text))
